/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file     speedrefdata.cc
 * @author   Thorsten Spexard
 * @date     08.08.2005
 * @brief    Converting and extracting speedref data to and from xml-structure
 */

#include "SpeedData.h"
#include "../../units/UnitConverter.h"

using namespace std;
using namespace xmltio;
using namespace airobots;

SpeedUnit SpeedData::iSU = METER_PER_SEC;
AngleUnit SpeedData::iAU = RADIAN;

std::string SpeedData::getBaseTag() {
	return "SPEEDS";
}

std::string SpeedData::getTemplate() {
	return std::string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate() + MicroTimestamp::getTemplate()
			+ "<ROTATIONSPEED quant=\"\"/>" + "<TRANSLATIONSPEED quant=\"\"/>"
			+ "</" + getBaseTag() + ">";
}

SpeedData::SpeedData() :
		Type("unknown") {

}

double SpeedData::getTranslation(SpeedUnit u) const {
	return UnitConverter::convert(translation, iSU, u);
}

void SpeedData::setTranslation(double translation, SpeedUnit u) {
	this->translation = UnitConverter::convert(translation, u, iSU);
}

double SpeedData::getRotation(AngleUnit u) const {
	return UnitConverter::convert(rotation, iAU, u);
}

void SpeedData::setRotation(double rotation, AngleUnit u) {
	this->rotation = UnitConverter::convert(rotation, u, iAU);
}

void SpeedData::sanitizeLocation(xmltio::Location &loc) const {

}

void SpeedData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath(MicroTimestamp::getBaseTag())] = getReadTime();
	loc[XPath("ROTATIONSPEED/@quant")] = getRotation(iAU);
	loc[XPath("TRANSLATIONSPEED/@quant")] = getTranslation(iSU);

}

void SpeedData::fromLocation(const xmltio::Location &loc_, SpeedData &sd) {

	Type::fromLocation(loc_, sd);

	sd.setRotation(extract<double>(loc_[XPath("ROTATIONSPEED/@quant")]), iAU);
	sd.setTranslation(extract<double>(loc_[XPath("TRANSLATIONSPEED/@quant")]),
			iSU);

	sd.setReadTime(
			extract<MicroTimestamp>(loc_[XPath(MicroTimestamp::getBaseTag())]));

}
