/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.person;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.Type;

public class PerceptList<T extends Type> extends Type {

	private Class<T> perceptType;
	private List<T> percepts = new ArrayList<T>();

	private static Logger logger = Logger.getLogger(Type.class);

	public PerceptList(Class<T> type) {
		perceptType = type;
	}

	public PerceptList(Class<T> type, List<T> percepts) {
		perceptType = type;
		this.percepts = new ArrayList<T>(percepts);
	}

	public Class<T> getPerceptType() {
		return perceptType;
	}

	public List<T> getPercepts() {
		return percepts;
	}

	public void addPercept(T percept) {
		percepts.add(percept);
	}

	public static String getBaseTag() {
		return "PERCEPTS";
	}

	public void fillInto(Element parent) {
		super.fillInto(parent);

		for (T p : percepts) {

			try {

				String baseTag = (String) perceptType.getMethod("getBaseTag")
						.invoke(null);
				Element perceptElement = new Element(baseTag);
				p.fillInto(perceptElement);
				parent.appendChild(perceptElement);

			} catch (IllegalArgumentException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (SecurityException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (IllegalAccessException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (InvocationTargetException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (NoSuchMethodException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			}

		}
	}

	public static <T extends Type> void fromElement(Element element,
			PerceptList<T> type) {

		Type.fromElement(element, type);

		try {

			String baseTag = (String) type.getPerceptType()
					.getMethod("getBaseTag").invoke(null);
			Nodes nodes = element.query(baseTag);

			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);

				if (node instanceof Element) {
					Element perceptElement = (Element) node;

					T parsed = type.getPerceptType().newInstance();
					Method fromElement = type.getPerceptType()
							.getMethod(
									"fromElement",
									new Class[] { Element.class,
											type.getPerceptType() });
					fromElement.invoke(null, perceptElement, parsed);

					type.addPercept(parsed);
				}
			}

		} catch (InstantiationException e) {
			logger.fatal("Reflection error. Types must have a default constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
		} catch (IllegalAccessException e) {
			logger.fatal("Reflection error. Types must have a default constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
		} catch (SecurityException e) {
			logger.fatal("Reflection error. SecurityException");
			logger.debug(e);
			assert false : "Reflection error. SecurityException";
		} catch (NoSuchMethodException e) {
			logger.fatal("Reflection error. NoSuchMethodException");
			logger.debug(e);
			assert false : "Reflection error.";
		} catch (IllegalArgumentException e) {
			logger.fatal("Reflection error. IllegalArgumentException");
			logger.debug(e);
			assert false : "Reflection error. IllegalArgumentException";
		} catch (InvocationTargetException e) {
			logger.fatal("Reflection error. InvocationTargetException");
			logger.debug(e);
			assert false : "Reflection error. InvocationTargetException";
		}
	}
}
