/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.speechrec;

import java.util.LinkedList;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.Type;

/**
 * @author hterhors
 */
public class GPSRActionList extends Type {

	private static Logger logger = Logger.getLogger(GPSRActionList.class);

	private LinkedList<GPSRAction> actions = new LinkedList<GPSRAction>();

	/** Constructor. */
	public GPSRActionList() {
	}

	public void addAction(GPSRAction action) {
		actions.add(action);
	}

	public LinkedList<GPSRAction> getActions() {
		return actions;
	}

	public static String getBaseTag() {
		return "GPSRACTIONLIST";
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);
		for (GPSRAction action : actions) {
			Element actionElement = new Element(GPSRAction.getBaseTag());
			action.fillInto(actionElement);
			parent.appendChild(actionElement);
		}

	}

	/** @{inheritDoc */
	public static void fromElement(Element element, GPSRActionList list) {

		Nodes nodes = element.query(GPSRAction.getBaseTag());
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {

			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element elem = (Element) node;

				GPSRAction action = new GPSRAction();
				GPSRAction.fromElement(elem, action);

				list.addAction(action);

			}

		}

	}

}
