/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.map;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.map.DynamicGridMap;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class DynamicGridMapTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		DynamicGridMap original = new DynamicGridMap();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setHeight(23445);
		original.setOriginX(-345);
		original.setOriginY(123);
		original.setResolution(0.034,LengthUnit.METER);
		original.setUri("blubber/bla");
		original.setWidth(34539);

		Document doc = new Document(new Element(DynamicGridMap.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		DynamicGridMap parsed = new DynamicGridMap();
		DynamicGridMap.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getHeight(), parsed.getHeight());
		assertEquals(original.getOriginX(), parsed.getOriginX());
		assertEquals(original.getOriginY(), parsed.getOriginY());
		assertEquals(original.getResolution(LengthUnit.METER),
				parsed.getResolution(LengthUnit.METER), 0.00001);
		assertEquals(original.getUri(), parsed.getUri());
		assertEquals(original.getWidth(), parsed.getWidth());

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("DynamicGridMap.xml"));

		DynamicGridMap parsed = new DynamicGridMap();
		DynamicGridMap.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(),
				DynamicGridMap.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(74, parsed.getOriginX());
		assertEquals(34, parsed.getOriginY());
		assertEquals(0.45, parsed.getResolution(LengthUnit.METER), 0.00001);
		assertEquals(745, parsed.getWidth());
		assertEquals(123, parsed.getHeight());
		assertEquals("test/uri", parsed.getUri());

	}

}
