/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/
package de.unibi.airobots.btl.data.vision3d;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.vision3d.KinectData;
import de.unibi.airobots.btl.units.AngleUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class KinectDataTest {

	private Logger logger = Logger.getLogger(getClass());
	private double delta = 0.0001;

	@Test
	public void selfCompatibility() throws Exception {

		KinectData original = new KinectData();
		original.setTilt(180.0, AngleUnit.DEGREE);
		original.setAcceleration(0.0, 1.0, 2.0);

		Document doc = new Document(new Element(KinectData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		KinectData parsed = new KinectData();
		KinectData.fromElement(doc.getRootElement(), parsed);

		assertEquals(Math.PI, parsed.getTilt(AngleUnit.RADIAN), delta);
		assertEquals(original.getTilt(AngleUnit.RADIAN),
				parsed.getTilt(AngleUnit.RADIAN), delta);

		assertEquals(original.getAccelerationX(), parsed.getAccelerationX(),
				delta);
		assertEquals(original.getAccelerationY(), parsed.getAccelerationY(),
				delta);
		assertEquals(original.getAccelerationZ(), parsed.getAccelerationZ(),
				delta);

		System.out.println(parsed);
	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser
				.build(TestUtils.makeTestFileName("KinectData.xml"));

		KinectData parsed = new KinectData();
		KinectData.fromElement(doc.getRootElement(), parsed);

		assertEquals(doc.getRootElement().getLocalName(),
				KinectData.getBaseTag());

		assertEquals(new Timestamp(253543797l, 253543712l),
				parsed.getTimestamp());
		assertEquals(180.0, parsed.getTilt(AngleUnit.DEGREE), delta);

		assertEquals(0.0, parsed.getAccelerationX(), 0.0001);
		assertEquals(1.0, parsed.getAccelerationY(), 0.0001);
		assertEquals(2.0, parsed.getAccelerationZ(), 0.0001);
	}
}
